<!-- deprecated, please use "cascade/generic/wrapper.html" -->{% load cms_tags %}
{% with instance_css_classes=instance.css_classes instance_inline_styles=instance.inline_styles %}
<section id="{{ instance.glossary.id }}">
{% for plugin in instance.child_plugin_instances %}
	{% render_plugin plugin %}
{% endfor %}
</section>
{% endwith %}