# -*- coding: utf-8 -*-
# Generated by Django 1.9.9 on 2016-09-15 16:22
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import filer.fields.file
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('filer', '0006_auto_20160623_1627'),
        ('cmsplugin_cascade', '0012_auto_20160619_1854'),
    ]

    operations = [
        migrations.CreateModel(
            name='IconFont',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('identifier', models.CharField(max_length=50, unique=True, verbose_name='Identifier')),
                ('config_data', jsonfield.fields.JSONField()),
                ('font_folder', models.FilePathField(allow_files=False, allow_folders=True, path='/Users/jrief/Workspace/upload/stofferia/media/icon_fonts')),
                ('zip_file', filer.fields.file.FilerFileField(on_delete=django.db.models.deletion.CASCADE, to='filer.File')),
            ],
            options={
                'verbose_name': 'Uploaded Icon Font',
                'verbose_name_plural': 'Uploaded Icon Fonts',
            },
        ),
    ]
