{% extends "cascade/admin/change_form.html" %}
{% load cms_tags %}

{% block after_field_sets %}
	{{ block.super }}
	<script type="text/javascript">
	django.cascade.iconfont_stylesheet_urls = {{% for icof in icon_fonts %}
		'{{ icof.id }}': '{{ icof.get_stylesheet_url }}'{% if not forloop.last %},{% endif %}{% endfor %}
	};
	django.cascade.fetch_fonticons_url = "{% cms_admin_url 'fetch_fonticons' %}";
	</script>
{% endblock %}
