{% extends "cascade/admin/change_form.html" %}
{% load cms_static %}

{% block extrahead %}
{{ block.super }}
<script src="{% static_with_version "cms/js/dist/bundle.admin.base.min.js" %}"></script>
<script>
(function($) {
// CMS.$ will be passed for $
$(document).ready(function () {
	CMS.API.Helpers.csrf('{{ csrf_token }}');
});
})(CMS.$);
</script>
{% endblock %}
